//===================================================== file = generate.c =====
//=  Traffic generator for VOQ switch simulation model                        =
//=============================================================================
//=  Notes:  1)                                                               =
//=---------------------------------------------------------------------------=
//=  Contains: generate() - Poisson with IEEE workgroup mix                   =
//=---------------------------------------------------------------------------=
//=  History:  KJC (01/27/02) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//============================================================================
//==  Function to generate IEEE workstation mix packets (Poisson arrivals)  ==
//=     - Based on "WG AVG" in Frazier document                             ==
//==    - Assume that "< 1600" is same as "< 1500" in Frazier document      ==
//==    - Mean packet length is 590.1 bytes                                 ==
//============================================================================
void generate1(int in_id)
{
  int    out_id;        // To (destination) outport number
  double org_time;      // Origination time of a packet
  int    pkt_len;       // Packet length pulled from IEEE dist
  double z;             // Random number between 0.0 and 1.0

  create("generate1");

  // Packet generation loop
  while(TRUE)
  {
    // Hold for desired interarrival time
    hold(expntl(1.0 / Lambda1));

    // Select packet size in bytes from empirical IEEE distribution
    z = uniform(0.0, 1.0);
    if (z < 0.197) pkt_len = 100;
    else if (z < 0.406) pkt_len = 200;
    else if (z < 0.464) pkt_len = 300;
    else if (z < 0.498) pkt_len = 400;
    else if (z < 0.519) pkt_len = 500;
    else if (z < 0.628) pkt_len = 600;
    else if (z < 0.647) pkt_len = 700;
    else if (z < 0.667) pkt_len = 800;
    else if (z < 0.687) pkt_len = 900;
    else if (z < 0.719) pkt_len = 1000;
    else if (z < 0.915) pkt_len = 1100;
    else if (z < 0.916) pkt_len = 1200;
    else if (z < 0.917) pkt_len = 1300;
    else if (z < 0.919) pkt_len = 1400;
    else if (z <= 1.000) pkt_len = 1500;

    // Put the packet into the input port
    out_id = random_int(0, (N - 1));
    org_time = clock;
    in[in_id](in_id, out_id, pkt_len, org_time);
  }
}

//============================================================================
//==  Function to generate "USF distribution" packets (Poisson arrivals)    ==
//==    - Mean packet length is 364.73 bytes                                ==
//==    - Packet type may only be 0 (low priority) or 1 (high priority)     ==
//============================================================================
void generate2(int in_id)
{
  int    out_id;        // To (destination) outport number
  int    pkt_len;       // Packet length pulled from IEEE dist
  double z;             // Random number between 0.0 and 1.0
  int    i;             // Loop counter
  double dist[] =
  {0.415253,0.415709,0.431188,0.431656,0.432044,0.432946,0.455365,0.455860,
   0.456605,0.457221,0.461922,0.462579,0.463085,0.463485,0.474875,0.475504,
   0.476076,0.476439,0.478190,0.478628,0.479206,0.479543,0.480358,0.480808,
   0.481356,0.481733,0.540933,0.541436,0.541755,0.541949,0.542910,0.543168,
   0.544672,0.548817,0.550535,0.551372,0.551555,0.552008,0.553670,0.554288,
   0.554474,0.554709,0.555293,0.555398,0.556675,0.560033,0.560574,0.563122,
   0.563288,0.563673,0.564388,0.564443,0.564500,0.565243,0.567481,0.567557,
   0.567717,0.568242,0.579780,0.579933,0.580024,0.580138,0.581891,0.582157,
   0.582273,0.582331,0.585249,0.585425,0.585640,0.585780,0.586924,0.587015,
   0.587100,0.587232,0.587629,0.587753,0.587893,0.587991,0.588326,0.588546,
   0.588735,0.588849,0.591462,0.591621,0.592102,0.592327,0.597746,0.597870,
   0.597962,0.598131,0.600220,0.600867,0.600952,0.601118,0.601830,0.601951,
   0.602129,0.602200,0.602945,0.603167,0.603239,0.603368,0.612495,0.612584,
   0.612700,0.612784,0.626867,0.627041,0.627234,0.627306,0.632005,0.632078,
   0.632264,0.632314,0.633885,0.633932,0.634039,0.634179,0.635973,0.636047,
   0.636115,0.636174,0.638329,0.638372,0.638476,0.638540,0.640353,0.640433,
   0.640579,0.640650,0.641472,0.641567,0.641660,0.642833,0.643512,0.644452,
   0.645050,0.645096,0.645372,0.645442,0.645510,0.645999,0.646181,0.646219,
   0.646475,0.646592,0.647015,0.647174,0.647248,0.647369,0.647822,0.647915,
   0.648028,0.648125,0.648852,0.648977,0.649015,0.649085,0.649450,0.649537,
   0.649602,0.649634,0.650080,0.650111,0.650140,0.650332,0.650573,0.650623,
   0.650677,0.650695,0.650881,0.650917,0.651067,0.651088,0.651345,0.651409,
   0.651505,0.651637,0.651781,0.651806,0.651882,0.651903,0.652106,0.652809,
   0.652856,0.652897,0.654575,0.654638,0.654741,0.654819,0.655113,0.655173,
   0.655295,0.655363,0.655497,0.655579,0.655664,0.655721,0.655932,0.656080,
   0.656185,0.656247,0.656447,0.656473,0.656546,0.656586,0.656686,0.656765,
   0.656837,0.656868,0.656944,0.657003,0.657026,0.657066,0.657129,0.657175,
   0.657204,0.657231,0.657504,0.657542,0.657564,0.657633,0.657858,0.657879,
   0.657914,0.657940,0.657986,0.658011,0.658049,0.658073,0.658206,0.658237,
   0.658274,0.658296,0.658515,0.658528,0.658541,0.658565,0.659489,0.659520,
   0.659548,0.659566,0.659644,0.659695,0.659795,0.659864,0.659942,0.660006,
   0.660040,0.660087,0.660557,0.660583,0.660624,0.660659,0.660709,0.660740,
   0.660771,0.660813,0.660889,0.660944,0.660994,0.661046,0.661129,0.661225,
   0.661267,0.661307,0.661407,0.661462,0.661537,0.661611,0.673249,0.673295,
   0.673365,0.673425,0.673503,0.673543,0.673674,0.673729,0.673818,0.675457,
   0.675512,0.675572,0.675917,0.676000,0.676066,0.676139,0.676229,0.676282,
   0.676361,0.676410,0.676501,0.676551,0.676600,0.676652,0.676778,0.676820,
   0.676849,0.676886,0.677023,0.677078,0.677109,0.677161,0.677215,0.677276,
   0.677315,0.677349,0.677433,0.677570,0.677659,0.677774,0.677894,0.677945,
   0.678019,0.678069,0.678315,0.678368,0.678402,0.678495,0.678791,0.678821,
   0.678864,0.678905,0.678992,0.679052,0.679104,0.679145,0.679294,0.679356,
   0.679400,0.679768,0.679995,0.680383,0.680694,0.680752,0.680943,0.680991,
   0.681066,0.681154,0.681206,0.681267,0.681331,0.681378,0.681460,0.681508,
   0.681570,0.681624,0.681760,0.681831,0.681896,0.681968,0.682140,0.682177,
   0.682220,0.682277,0.682368,0.682425,0.682502,0.682584,0.682697,0.682768,
   0.682929,0.683010,0.683178,0.683237,0.683298,0.683357,0.683434,0.683476,
   0.683528,0.683587,0.683764,0.683845,0.684041,0.684133,0.684255,0.684298,
   0.684337,0.684388,0.684527,0.684583,0.684619,0.684664,0.684727,0.684772,
   0.684803,0.684862,0.684932,0.684987,0.685036,0.685071,0.685181,0.685222,
   0.685256,0.685309,0.685366,0.685408,0.685486,0.685520,0.685590,0.685626,
   0.685666,0.685712,0.685768,0.691269,0.691300,0.691339,0.691432,0.691472,
   0.691513,0.691548,0.691625,0.691655,0.691690,0.691727,0.691768,0.691805,
   0.691831,0.691865,0.692457,0.692498,0.692533,0.692630,0.692661,0.692688,
   0.692735,0.692785,0.692915,0.692985,0.693090,0.693143,0.693256,0.693301,
   0.693329,0.693366,0.694090,0.694125,0.694162,0.694196,0.694238,0.694275,
   0.694291,0.694317,0.694391,0.694414,0.694447,0.694480,0.694561,0.694589,
   0.694627,0.694653,0.694688,0.694702,0.694721,0.704220,0.704244,0.704263,
   0.704278,0.704298,0.704334,0.704349,0.704364,0.704378,0.704407,0.704423,
   0.704438,0.711246,0.711329,0.711344,0.711359,0.711374,0.711400,0.711414,
   0.711425,0.711445,0.711591,0.711608,0.711622,0.711635,0.711690,0.711700,
   0.711721,0.711735,0.711756,0.711768,0.711782,0.711793,0.711845,0.711861,
   0.711882,0.711900,0.711946,0.711959,0.711969,0.711979,0.782396,0.782405,
   0.782418,0.782429,0.782444,0.782458,0.782471,0.782484,0.782505,0.782517,
   0.782530,0.782542,0.837785,0.837797,0.837809,0.837821,0.837869,0.837878,
   0.837888,0.837917,0.838085,0.838094,0.838109,0.838122,0.838145,0.838153,
   0.838165,0.838175,0.838204,0.838216,0.838228,0.838239,0.838262,0.838277,
   0.838292,0.838304,0.841695,0.841704,0.842162,0.842175,0.842235,0.842247,
   0.842258,0.842271,0.842318,0.842331,0.842344,0.842357,0.842381,0.842391,
   0.842409,0.842417,0.842444,0.845607,0.845625,0.845639,0.845681,0.845692,
   0.845697,0.845711,0.845746,0.845762,0.845774,0.845787,0.845820,0.845830,
   0.845843,0.845854,0.845946,0.845963,0.845977,0.845993,0.846025,0.846037,
   0.846048,0.846073,0.846155,0.846168,0.846181,0.846192,0.846281,0.846295,
   0.846310,0.846319,0.846404,0.846417,0.846433,0.846456,0.847360,0.847371,
   0.847380,0.847397,0.847539,0.847548,0.847564,0.847578,0.847858,0.847867,
   0.852539,0.852555,0.852769,0.852783,0.852794,0.852801,0.852857,0.852867,
   0.852879,0.853874,0.853969,0.853982,0.853996,0.854015,0.854053,0.854067,
   0.854081,0.854098,0.854127,0.854137,0.854146,0.854156,0.854314,0.854338,
   0.854353,0.854366,0.854406,0.854440,0.854447,0.854456,0.854481,0.854492,
   0.854502,0.854528,0.854547,0.854557,0.855028,0.855048,0.855093,0.855102,
   0.855143,0.855155,0.855199,0.855213,0.855225,0.855233,0.855292,0.855304,
   0.855318,0.855327,0.855467,0.855474,0.855481,0.855488,0.855927,0.855939,
   0.855948,0.855955,0.856015,0.856028,0.856037,0.856045,0.856078,0.856089,
   0.856101,0.856109,0.856135,0.856146,0.856154,0.856162,0.856217,0.856231,
   0.856240,0.856254,0.856296,0.856302,0.856313,0.856321,0.856341,0.856352,
   0.856362,0.856369,0.856382,0.856392,0.856406,0.856412,0.856431,0.856439,
   0.856454,0.856463,0.856484,0.856491,0.856504,0.856513,0.856527,0.856537,
   0.856551,0.856560,0.856572,0.856580,0.856597,0.856603,0.856633,0.856644,
   0.856655,0.856664,0.856696,0.856705,0.856715,0.856723,0.856741,0.856755,
   0.856769,0.856807,0.856858,0.856868,0.856886,0.856900,0.856927,0.856936,
   0.856953,0.856963,0.856989,0.856999,0.857013,0.857021,0.857034,0.857041,
   0.857055,0.857064,0.857096,0.857103,0.857120,0.857129,0.857149,0.857157,
   0.857180,0.857185,0.857212,0.857218,0.857235,0.857243,0.857270,0.857278,
   0.857297,0.857307,0.857321,0.857327,0.857339,0.857351,0.857428,0.857440,
   0.857456,0.857464,0.857483,0.857490,0.857504,0.857513,0.857528,0.857536,
   0.857555,0.857562,0.857590,0.857598,0.857614,0.857620,0.857634,0.857641,
   0.857651,0.857657,0.857686,0.857695,0.857711,0.857717,0.857737,0.857745,
   0.857762,0.857775,0.857788,0.857797,0.857810,0.857818,0.857847,0.857860,
   0.857874,0.857881,0.857899,0.857906,0.857916,0.857933,0.857944,0.857950,
   0.857961,0.857969,0.857994,0.858001,0.858058,0.858065,0.858078,0.858088,
   0.858103,0.858112,0.858139,0.858145,0.858155,0.858164,0.858183,0.858193,
   0.858202,0.858214,0.858238,0.858243,0.858253,0.858261,0.858346,0.858353,
   0.858360,0.858368,0.858422,0.858430,0.858436,0.858446,0.858480,0.858486,
   0.858494,0.858501,0.858519,0.858529,0.858539,0.858548,0.858559,0.858564,
   0.858576,0.858583,0.858626,0.858637,0.858644,0.858655,0.858676,0.858684,
   0.858690,0.858697,0.858711,0.858719,0.858728,0.858737,0.858751,0.858757,
   0.858765,0.858770,0.858784,0.858791,0.858805,0.858821,0.858832,0.858841,
   0.858851,0.858859,0.858876,0.858888,0.858899,0.858911,0.858925,0.858931,
   0.858941,0.858954,0.859001,0.859011,0.859019,0.859025,0.859035,0.859044,
   0.859052,0.859061,0.859080,0.859087,0.859096,0.859110,0.859122,0.859132,
   0.859142,0.864516,0.864540,0.864547,0.864554,0.864567,0.864581,0.864589,
   0.864600,0.864610,0.864621,0.864629,0.864636,0.864648,0.869630,0.869641,
   0.869648,0.869653,0.869684,0.869689,0.869698,0.869705,0.869741,0.869747,
   0.869756,0.869763,0.869772,0.869779,0.869788,0.869796,0.869811,0.869816,
   0.869825,0.869833,0.869866,0.869878,0.869886,0.869894,0.869900,0.869905,
   0.869914,0.869921,0.869942,0.869949,0.869958,0.869962,0.869974,0.869979,
   0.869985,0.869996,0.870020,0.870026,0.870030,0.870038,0.870060,0.870064,
   0.870070,0.870075,0.870084,0.870090,0.870097,0.870106,0.870118,0.870125,
   0.871462,0.871473,0.871488,0.871497,0.871503,0.871509,0.871522,0.871528,
   0.871538,0.871545,0.871579,0.871588,0.871594,0.871609,0.871652,0.871659,
   0.871667,0.871672,0.871709,0.871716,0.871722,0.871730,0.871744,0.871752,
   0.871757,0.871765,0.871801,0.871807,0.871812,0.871819,0.871841,0.871849,
   0.871855,0.871862,0.871880,0.871887,0.871893,0.871902,0.871912,0.871920,
   0.871925,0.871932,0.871943,0.871953,0.871966,0.871975,0.871986,0.871995,
   0.872005,0.872011,0.872034,0.872043,0.872049,0.872059,0.872071,0.872079,
   0.872086,0.872092,0.872106,0.872112,0.872121,0.872125,0.872135,0.872141,
   0.872146,0.872153,0.872164,0.872173,0.872182,0.872190,0.872201,0.872207,
   0.872211,0.872218,0.872228,0.872233,0.872240,0.872245,0.872321,0.872324,
   0.872328,0.872337,0.872514,0.872524,0.872530,0.872535,0.872593,0.872597,
   0.872602,0.872611,0.872634,0.872644,0.872656,0.872665,0.872704,0.872709,
   0.872714,0.872720,0.872733,0.872739,0.872745,0.872757,0.872767,0.872778,
   0.872788,0.872795,0.872811,0.872815,0.872821,0.872826,0.872847,0.872854,
   0.872859,0.872867,0.872907,0.872924,0.872940,0.872950,0.872960,0.872965,
   0.872971,0.872977,0.872985,0.872991,0.872998,0.873004,0.873026,0.873030,
   0.873033,0.873042,0.873061,0.873066,0.873068,0.873072,0.873086,0.873091,
   0.873100,0.873104,0.873120,0.873125,0.873133,0.873136,0.873638,0.873642,
   0.873648,0.873655,0.873706,0.873710,0.873720,0.873726,0.873739,0.873745,
   0.873747,0.873750,0.873762,0.873767,0.873782,0.873790,0.873807,0.873809,
   0.873811,0.873815,0.873874,0.873879,0.873897,0.873900,0.873913,0.873916,
   0.873924,0.873932,0.873947,0.873950,0.873953,0.873958,0.873983,0.873986,
   0.873995,0.873999,0.874042,0.874046,0.874049,0.874053,0.874076,0.874082,
   0.874089,0.874093,0.874111,0.874113,0.874117,0.874120,0.874146,0.874151,
   0.874153,0.874156,0.874328,0.874331,0.874334,0.874337,0.874347,0.874351,
   0.874360,0.874363,0.874369,0.874372,0.874379,0.874382,0.874388,0.874390,
   0.874392,0.874395,0.874412,0.874416,0.874421,0.874425,0.874433,0.874436,
   0.874440,0.874443,0.874454,0.874456,0.874460,0.874467,0.874475,0.874483,
   0.874488,0.874493,0.874500,0.874504,0.874507,0.874509,0.874517,0.874520,
   0.874524,0.874527,0.879611,0.879621,0.879649,0.879652,0.879959,0.879963,
   0.879966,0.879972,0.880009,0.880018,0.880026,0.880031,0.880038,0.880041,
   0.880050,0.880055,0.880062,0.880067,0.880072,0.880079,0.880091,0.880097,
   0.880108,0.880124,0.880134,0.880140,0.880144,0.880149,0.880155,0.880161,
   0.880167,0.880171,0.880179,0.880182,0.880186,0.880196,0.880317,0.880320,
   0.880325,0.880329,0.880353,0.880359,0.880367,0.880372,0.880378,0.880383,
   0.880388,0.880391,0.880401,0.880407,0.880413,0.880419,0.880466,0.880471,
   0.880478,0.880483,0.880492,0.880497,0.880504,0.880508,0.880518,0.880525,
   0.880532,0.880607,0.881165,0.881173,0.881178,0.881184,0.881214,0.881219,
   0.881226,0.881231,0.881246,0.881249,0.881252,0.881255,0.881263,0.881266,
   0.881269,0.881272,0.881280,0.881287,0.881288,0.881292,0.881302,0.881309,
   0.881319,0.881331,0.884835,0.884846,0.884854,0.884862,0.884873,0.884885,
   0.884898,0.884912,0.885371,0.885377,0.885387,0.885395,0.890132,0.890136,
   0.890141,0.890146,0.890162,0.890167,0.890176,0.890179,0.890191,0.890193,
   0.890198,0.890203,0.890209,0.890212,0.890216,0.890220,0.890237,0.890243,
   0.890245,0.890247,0.890261,0.890266,0.890271,0.890285,0.890303,0.890306,
   0.890312,0.890318,0.890336,0.890341,0.890348,0.890353,0.890363,0.890366,
   0.890370,0.890376,0.890392,0.890395,0.890399,0.890404,0.890409,0.890414,
   0.890421,0.890425,0.890434,0.890439,0.890443,0.890446,0.890451,0.890452,
   0.890459,0.890462,0.890470,0.890477,0.890481,0.890483,0.890489,0.890493,
   0.890495,0.890500,0.890512,0.890521,0.890526,0.890528,0.890533,0.890536,
   0.890541,0.890543,0.890557,0.890560,0.890563,0.890566,0.890576,0.890577,
   0.890588,0.890591,0.890895,0.890901,0.890904,0.890907,0.890915,0.890919,
   0.890923,0.890930,0.891104,0.891109,0.891115,0.891121,0.891127,0.891133,
   0.891136,0.891141,0.891148,0.891151,0.891153,0.891158,0.891166,0.891170,
   0.891175,0.891182,0.891195,0.891197,0.891199,0.891209,0.892012,0.892018,
   0.892021,0.892026,0.892034,0.892038,0.892043,0.892049,0.892054,0.892061,
   0.892064,0.892069,0.892075,0.892078,0.892081,0.892083,0.892086,0.892089,
   0.892095,0.892098,0.892104,0.892108,0.892112,0.892115,0.892122,0.892126,
   0.892130,0.892132,0.892146,0.892149,0.892350,0.892353,0.892359,0.892368,
   0.892836,0.892839,0.892869,0.892872,0.892875,0.892880,0.892891,0.892893,
   0.892897,0.892900,0.892912,0.892918,0.892923,0.892927,0.892934,0.892937,
   0.892941,0.892945,0.892960,0.892963,0.892969,0.892972,0.892977,0.892982,
   0.892989,0.892991,0.893036,0.893039,0.893041,0.893044,0.893050,0.893054,
   0.893056,0.893061,0.893174,0.893178,0.893184,0.893186,0.893203,0.893209,
   0.893212,0.893215,0.916882,0.916884,0.916885,0.916889,1.000000};

  create("generate2");

  // Packet generation loop
  while(TRUE)
  {
    // Hold for desired interarrival time
    hold(expntl(1.0 / Lambda2));

    // Choose packet size in bytes from empirical USF distribution
    z = uniform(0.0, 1.0);
    for (i=0; i<(1518 - 64); i++)
      if (z <= dist[i]) break;
    pkt_len = i + 64;

    // Choose the output port
    out_id = random_int(0, (N - 1));

    // Send the packet
    in[in_id](in_id, out_id, pkt_len, clock);
  }
}
