//======================================================= file = inport.c =====
//=  VOQ input port for VOQ switch simulation model                           =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: voq_arbit_rr() - VOQ arbitration by round robin                =
//=            voq_arbit_opf() - VOQ arbitration by OPF                       =
//=            inport() - Input port containing VOQ buffers                   =
//=---------------------------------------------------------------------------=
//=  History:  KJC (01/27/02) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//=============================================================================
//==  Poller for input port VOQ buffers                                      ==
//==   - RR (next poll starts one index past selected queue to tie break)    ==
//=============================================================================
void voq_arbit_rr(int in_id)
{
  int      out_id;              // Output port id
  int      out_poll;            // Output port id to be polled
  int      flag;                // Flag variable
  int      i;                   // Loop counter

  create("voq_arbit_rr");

  // Poll input port VOQ buffers forever
  out_id = 0;
  Count_in[in_id] = Burst_in;
  while(1)
  {
    // Test if any VOQ buffer transfers can be done
    flag = FALSE;
    for (i=0; i<N; i++)
      if ((queue_cnt(VOQ_first[in_id][i]) > 0) &&
          (queue_cnt(CP_event[in_id][i]) == 0))
        flag = TRUE;

    // If no transfer can be done, wait for an input port state change
    if (flag == FALSE) wait(State_change[in_id]);

    // RR poll to try to find the first VOQ that can be transferred
    for (i=0; i<N; i++)
    {
      out_poll = (out_id + i) % N;
      if ((queue_cnt(VOQ_first[in_id][out_poll]) > 0) &&
          (queue_cnt(CP_event[in_id][out_poll]) == 0))
      {
        set(VOQ_first[in_id][out_poll]);
        hold((8.0 * Pkt_len[in_id][out_poll]) / MEDIA_RATE);
        if ((Level[in_id][out_poll] > Thresh) && (Count_in[in_id] > 0))
        {
          out_id = out_id;
        }
        else
        {
          out_id = (out_poll + 1) % N;
          Count_in[in_id] = Burst_in;
        }
        break;
      }
    }
  }
}

//=============================================================================
//==  Poller for input port VOQ buffers                                      ==
//==   - OPF (uses floating point time-stamp comparisons)                    ==
//=============================================================================
void voq_arbit_opf(int in_id)
{
  double   oldest;              // Oldest timestamp value
  int      out_id;              // Output port id
  int      flag;                // Flag variable
  int      i;                   // Loop counter

  create("voq_arbit_opf");

  // Poll input port VOQ buffers forever
  while(1)
  {
    // Test if any VOQ buffer transfers can be done
    flag = FALSE;
    for (i=0; i<N; i++)
      if ((queue_cnt(VOQ_first[in_id][i]) > 0) &&
          (queue_cnt(CP_event[in_id][i]) == 0))
        flag = TRUE;

    // If no transfer can be done, wait for an input port state change
    if (flag == FALSE) wait(State_change[in_id]);

    // Poll to try to find oldest packet
    oldest = clock;
    out_id = -1;
    for (i=0; i<N; i++)
    {
      if ((queue_cnt(VOQ_first[in_id][i]) > 0) &&
          (queue_cnt(CP_event[in_id][i]) == 0))
      {
        if (Time_stamp[in_id][i] <= oldest)
        {
          oldest = Time_stamp[in_id][i];
          out_id = i;
        }
      }
    }

    // If an eligible VOQ was found, transfer the VOQ contents to the CP buffer
    if (out_id >= 0)
    {
      set(VOQ_first[in_id][out_id]);
      hold((8.0 * Pkt_len[in_id][out_id]) / MEDIA_RATE);
    }
  }
}

//=============================================================================
//==  Input port containing VOQ buffers                                      ==
//=============================================================================
void inport(int in_id, int out_id, int pkt_len, double org_time)
{
  create("inport");

  // If destination buffer is empty then set the Arrival event
  if (queue_cnt(VOQ_first[in_id][out_id]) == 0) set(State_change[in_id]);

  // Queue the arriving packet in the appropriate VOQ buffer
  Level[in_id][out_id] = Level[in_id][out_id] + pkt_len;
  queue(VOQ_event[in_id][out_id]);
  Time_stamp[in_id][out_id] = org_time;
  Pkt_len[in_id][out_id] = pkt_len;
  assert(queue_cnt(VOQ_first[in_id][out_id]) == 0);
  queue(VOQ_first[in_id][out_id]);
  set(VOQ_event[in_id][out_id]);

  // Forward the packet to the crosspoint buffer
  out[out_id](in_id, out_id, pkt_len, org_time);
  set(Xfer_cp[out_id]);
  set(Xfer_voq[in_id]);

  // Update some common variables
  Level[in_id][out_id] = Level[in_id][out_id] - pkt_len;
  Count_in[in_id] = Count_in[in_id] - pkt_len;

  return;
}
