//========================================================= file = main.c =====
//=  Main program for a VOQ switch simulation model                           =
//=   - Implements variable length packets and buffered crossbar              =
//=============================================================================
//=  Notes:  1) Simulation output is written to OUTFILE defined in common.h   =
//=---------------------------------------------------------------------------=
//=  Build:  Build as a CSIM application linking to appropriate CSIM libs.    =
//=          Component files are:                                             =
//=            - common.h, main.c, generate.c, inport.c, outport.c            =
//=---------------------------------------------------------------------------=
//=  History:  KJC (08/12/01) - Genesis                                       =
//=============================================================================
#define MAIN                    // This is main (used in common.h)

//----- Include files ---------------------------------------------------------
#include <stdio.h>              // Needed for printf() and fprintf()
#include <stdlib.h>             // Needed for exit()
#include <math.h>               // Needed for sqrt()
#include "csim.h"               // Needed for CSIM
#include "common.h"             // Needed for common stuff

//=============================================================================
//==  Main program where sim() is the first process for a CSIM simulation    ==
//=============================================================================
void sim(void)
{
  FILE     *fp;                 // File pointer for output file
  int      run_num;             // Simulation run number
  double   global_mean;         // Overall (global) mean delay
  double   global_std_dev;      // Overall (global) std dev delay
  int      i, j;                // Generic loop counters

  // Set model name for CSIM
  set_model_name("VOQ switch simulation model");

  // Increase maximum number of allowed processes, facilities, and servers
  max_facilities(1000L);
  max_servers(10000L);
  max_processes(1000000L);
  max_events(1000000L);

  // Open output file for writing (delete and re-write if file already exists)
  fp = fopen(OUT_FILE, "w");
  if (fp == NULL)
  {
    printf("  >>> ERROR in creating output file '%s' \n", OUT_FILE);
    exit(1);
  }

  // Output banner to stdout and output file
  printf("*** BEGIN SIMULATION -- Version %s *** \n", VERSION);
  fprintf(fp,"*** BEGIN SIMULATION -- Version %s *** \n", VERSION);

  // *** Main simulation loop ***
  for (run_num=0; run_num<1; run_num++)
  {
    create("sim");

    // Initialize run parameters
    Sim_time = 100.0;
    Thresh = 32 * 1500;
    Burst_in = Thresh;
    Burst_out = Burst_in;
    Lambda1 = 0.98 * 2118.2850;
    Lambda2 = 0.98 * 3427.19;

    // Assign pointers to generators, arbiters, inports, and outports
    for (i=0; i<N; i++)
    {
      if (GEN_FLAG == 1) gen[i] = generate1;
      if (GEN_FLAG == 2) gen[i] = generate2;
      if (SCHED_FLAG == 1) voq_arb[i] = voq_arbit_rr;
      if (SCHED_FLAG == 2) voq_arb[i] = voq_arbit_opf;
      cp_arb[i] = cp_arbit;
      in[i] = inport;
      out[i] = outport;
    }

    // Variable initializations
    for (i=0; i<N; i++)
      for (j=0; j<N; j++)
        Level[i][j] = 0;

    // CSIM initializations
    for (i=0; i<N; i++)
    {
      Out_link[i] = facility("Output port link");
      State_change[i] = event("State change to an input port buffer");
      Xfer_voq[i] = event("VOQ-to-CP buffer xfer done for an in_id");
      Xfer_cp[i] = event("VOQ-to-CP buffer xfer done for an out_id");
      Xmit_done[i] = event("CP buffer to link transmit done event");
      Delay_table[i] = table("Packet delay table");
      Bytes_table[i] = table("Bytes transfered table");
      for (j=0; j<N; j++)
      {
        VOQ_first[i][j] = event("Input port VOQ HOL for packets");
        VOQ_event[i][j] = event("Input port VOQ buffers for packets");
        set(VOQ_event[i][j]);
        CP_event[i][j] = event("Cross point buffers for outports");
      }
    }

    // Kick-off the generators
    for (i=0; i<N; i++)
      gen[i](i);

    // Kick-off the arbiters
    for (i=0; i<N; i++)
    {
      voq_arb[i](i);
      cp_arb[i](i);
    }

    // Run until Sim_time expires while outputting status messages to stdout
    run_sim(Sim_time, run_num);

    // Compute global statistics
    global_mean = global_std_dev = 0.0;
    for (i=0; i<N; i++)
    {
      global_mean = global_mean + table_mean(Delay_table[i]) / N;
      global_std_dev = global_std_dev + sqrt(table_var(Delay_table[i])) / N;
    }

    // Output simulation configuration parameters
    fprintf(fp,"&**************************************************\n");
    fprintf(fp,"&**   Results from VOQ switch simulation model   **\n");
    fprintf(fp,"&**     >>> Polled buffered crosspoints <<<      **\n");
    fprintf(fp,"&**************************************************\n");
    if (SCHED_FLAG == 1)
      fprintf(fp,"&**  >>> VOQ RR scheduling                 \n");
    if (SCHED_FLAG == 2)
      fprintf(fp,"&**  >>> VOQ OPF scheduling                \n");
    fprintf(fp,"&** Simulation time      = %9.0f ms          \n",
      Sim_time);
    fprintf(fp,"&** Media rate           = %9.0f Mbps        \n",
      0.001 * MEDIA_RATE);
    fprintf(fp,"&** Switch size          = %d ports          \n",
      N);
    fprintf(fp,"&** Speed-up             = %f                \n",
      SPEED_UP);
    fprintf(fp,"&** Output burst         = %d bytes          \n",
      Burst_out);
    if (SCHED_FLAG == 1)
    {
      fprintf(fp,"&** Threshold            = %d bytes          \n",
        Thresh);
      fprintf(fp,"&** Input burst          = %d bytes          \n",
        Burst_in);
    }
    fprintf(fp,"&************************************************\n");

    // Output per port statistics
    for (i=0; i<N; i++)
    {
      fprintf(fp,"%2d===============================================\n", i);
      fprintf(fp,"%2d= Outport utilization  = %5.2f %%       \n", i,
        100.0 * (8.0 * table_sum(Bytes_table[i])) / (MEDIA_RATE * Sim_time));
      fprintf(fp,"%2d= Mean delay           = %f ms          \n", i,
        table_mean(Delay_table[i]));
      fprintf(fp,"%2d= Std Dev delay        = %f ms          \n", i,
        sqrt(table_var(Delay_table[i])));
      fprintf(fp,"%2d= Max delay            = %f ms          \n", i,
        table_max(Delay_table[i]));
    }

    // Output global statistics
    fprintf(fp,"&&================================================\n");
    fprintf(fp,"&&  Global mean delay    = %f ms         \n", global_mean);
    fprintf(fp,"&&  Global std dev delay = %f ms         \n", global_std_dev);
    fprintf(fp,"&&================================================\n");

    // Reset everything for a possible rerun
    rerun();
    reset_prob(1L);
  }

  // Close and end
  printf("*** END OF SIMULATION *** \n");
  fprintf(fp,"*** END OF SIMULATION *** \n");
  fclose(fp);
}
